@@ -88,6 +88,12 @@ meta_ui_get_display (void)
   return GDK_DISPLAY_XDISPLAY (gdk_display_get_default ());
 }
 
+gint
+meta_ui_get_screen_number (void)
+{
+  return gdk_screen_get_number (gdk_screen_get_default ());
+}
+
 /* We do some of our event handling in frames.c, which expects
  * GDK events delivered by GTK+.  However, since the transition to
  * client side windows, we can't let GDK see button events, since the
@@ -291,7 +297,8 @@ meta_ui_new (Display *xdisplay,
   g_assert (gdisplay == gdk_display_get_default ());
 
   g_assert (xdisplay == GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()));
-  ui->frames = meta_frames_new (XScreenNumberOfScreen (screen));
+  ui->frames = meta_frames_new ();
+
   /* GTK+ needs the frame-sync protocol to work in order to properly
    * handle style changes. This means that the dummy widget we create
    * to get the style for title bars actually needs to be mapped
@@ -348,17 +355,15 @@ set_background_none (Display *xdisplay,
 }
 
 Window
-meta_ui_create_frame_window (MetaUI *ui,
+meta_ui_create_frame_window (MetaUI  *ui,
                              Display *xdisplay,
-                             Visual *xvisual,
-			     gint x,
-			     gint y,
-			     gint width,
-			     gint height,
-			     gint screen_no)
+                             Visual  *xvisual,
+                             gint     x,
+                             gint     y,
+                             gint     width,
+                             gint     height)
 {
-  GdkDisplay *display = gdk_x11_lookup_xdisplay (xdisplay);
-  GdkScreen *screen = gdk_display_get_screen (display, screen_no);
+  GdkScreen *screen = gdk_screen_get_default ();
   GdkWindowAttr attrs;
   gint attributes_mask;
   GdkWindow *window;
